@@ -1,4 +1,4 @@
-# Copyright 2012-2019 The Meson development team
+# Copyrighs 2012-2019 The Meson development team
 
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -19,36 +19,34 @@ from itertools import chain
 from pathlib import PurePath
 from collections import OrderedDict
 from .mesonlib import (
-    MesonException, MachineChoice, PerMachine,
+    MesonException, MachineChoice, PerMachine, OrderedSet,
     default_libdir, default_libexecdir, default_prefix, split_args
 )
+from .envconfig import get_env_var_pair
 from .wrap import WrapMode
 import ast
 import argparse
 import configparser
-from typing import (
-    Any, Dict, Generic, Iterable, Iterator, List, MutableMapping, Optional, Tuple, Type, TypeVar, Union,
-    TYPE_CHECKING,
-)
 import enum
 import shlex
+import typing as T
 
-if TYPE_CHECKING:
+if T.TYPE_CHECKING:
     from . import dependencies
     from .compilers import Compiler  # noqa: F401
     from .environment import Environment
 
-    OptionDictType = Dict[str, 'UserOption[Any]']
+    OptionDictType = T.Dict[str, 'UserOption[T.Any]']
 
-version = '0.53.0'
+version = '0.54.0'
 backendlist = ['ninja', 'vs', 'vs2010', 'vs2015', 'vs2017', 'vs2019', 'xcode']
 
 default_yielding = False
 
 # Can't bind this near the class method it seems, sadly.
-_T = TypeVar('_T')
+_T = T.TypeVar('_T')
 
-class UserOption(Generic[_T]):
+class UserOption(T.Generic[_T]):
     def __init__(self, description, choices, yielding):
         super().__init__()
         self.choices = choices
@@ -65,7 +63,7 @@ class UserOption(Generic[_T]):
     # Check that the input is a valid value and return the
     # "cleaned" or "native" version. For example the Boolean
     # option could take the string "true" and return True.
-    def validate_value(self, value: Any) -> _T:
+    def validate_value(self, value: T.Any) -> _T:
         raise RuntimeError('Derived option class did not override validate_value.')
 
     def set_value(self, newvalue):
@@ -99,11 +97,12 @@ class UserBooleanOption(UserOption[bool]):
         raise MesonException('Value %s is not boolean (true or false).' % value)
 
 class UserIntegerOption(UserOption[int]):
-    def __init__(self, description, min_value, max_value, value, yielding=None):
+    def __init__(self, description, value, yielding=None):
+        min_value, max_value, default_value = value
         super().__init__(description, [True, False], yielding)
         self.min_value = min_value
         self.max_value = max_value
-        self.set_value(value)
+        self.set_value(default_value)
         c = []
         if min_value is not None:
             c.append('>=' + str(min_value))
@@ -128,9 +127,9 @@ class UserIntegerOption(UserOption[int]):
         except ValueError:
             raise MesonException('Value string "%s" is not convertible to an integer.' % valuestring)
 
-class UserUmaskOption(UserIntegerOption, UserOption[Union[str, int]]):
+class UserUmaskOption(UserIntegerOption, UserOption[T.Union[str, int]]):
     def __init__(self, description, value, yielding=None):
-        super().__init__(description, 0, 0o777, value, yielding)
+        super().__init__(description, (0, 0o777, value), yielding)
         self.choices = ['preserve', '0000-0777']
 
     def printable_value(self):
@@ -150,7 +149,7 @@ class UserUmaskOption(UserIntegerOption, UserOption[Union[str, int]]):
             raise MesonException('Invalid mode: {}'.format(e))
 
 class UserComboOption(UserOption[str]):
-    def __init__(self, description, choices: List[str], value, yielding=None):
+    def __init__(self, description, choices: T.List[str], value, yielding=None):
         super().__init__(description, choices, yielding)
         if not isinstance(self.choices, list):
             raise MesonException('Combo choices must be an array.')
@@ -165,14 +164,14 @@ class UserComboOption(UserOption[str]):
             raise MesonException('Value "%s" for combo option is not one of the choices. Possible choices are: %s.' % (value, optionsstring))
         return value
 
-class UserArrayOption(UserOption[List[str]]):
+class UserArrayOption(UserOption[T.List[str]]):
     def __init__(self, description, value, split_args=False, user_input=False, allow_dups=False, **kwargs):
         super().__init__(description, kwargs.get('choices', []), yielding=kwargs.get('yielding', None))
         self.split_args = split_args
         self.allow_dups = allow_dups
         self.value = self.validate_value(value, user_input=user_input)
 
-    def validate_value(self, value, user_input: bool = True) -> List[str]:
+    def validate_value(self, value, user_input: bool = True) -> T.List[str]:
         # User input is for options defined on the command line (via -D
         # options). Users can put their input in as a comma separated
         # string, but for defining options in meson_options.txt the format
@@ -229,16 +228,16 @@ class UserFeatureOption(UserComboOption):
         return self.value == 'auto'
 
 
-def load_configs(filenames: List[str]) -> configparser.ConfigParser:
+def load_configs(filenames: T.List[str]) -> configparser.ConfigParser:
     """Load configuration files from a named subdirectory."""
-    config = configparser.ConfigParser()
+    config = configparser.ConfigParser(interpolation=None)
     config.read(filenames)
     return config
 
 
-if TYPE_CHECKING:
-    CacheKeyType = Tuple[Tuple[Any, ...], ...]
-    SubCacheKeyType = Tuple[Any, ...]
+if T.TYPE_CHECKING:
+    CacheKeyType = T.Tuple[T.Tuple[T.Any, ...], ...]
+    SubCacheKeyType = T.Tuple[T.Any, ...]
 
 
 class DependencyCacheType(enum.Enum):
@@ -262,7 +261,7 @@ class DependencySubCache:
 
     def __init__(self, type_: DependencyCacheType):
         self.types = [type_]
-        self.__cache = {}  # type: Dict[SubCacheKeyType, dependencies.Dependency]
+        self.__cache = {}  # type: T.Dict[SubCacheKeyType, dependencies.Dependency]
 
     def __getitem__(self, key: 'SubCacheKeyType') -> 'dependencies.Dependency':
         return self.__cache[key]
@@ -273,7 +272,7 @@ class DependencySubCache:
     def __contains__(self, key: 'SubCacheKeyType') -> bool:
         return key in self.__cache
 
-    def values(self) -> Iterable['dependencies.Dependency']:
+    def values(self) -> T.Iterable['dependencies.Dependency']:
         return self.__cache.values()
 
 
@@ -285,12 +284,12 @@ class DependencyCache:
     successfully lookup by providing a simple get/put interface.
     """
 
-    def __init__(self, builtins_per_machine: PerMachine[Dict[str, UserOption[Any]]], for_machine: MachineChoice):
-        self.__cache = OrderedDict()  # type: MutableMapping[CacheKeyType, DependencySubCache]
+    def __init__(self, builtins_per_machine: PerMachine[T.Dict[str, UserOption[T.Any]]], for_machine: MachineChoice):
+        self.__cache = OrderedDict()  # type: T.MutableMapping[CacheKeyType, DependencySubCache]
         self.__builtins_per_machine = builtins_per_machine
         self.__for_machine = for_machine
 
-    def __calculate_subkey(self, type_: DependencyCacheType) -> Tuple[Any, ...]:
+    def __calculate_subkey(self, type_: DependencyCacheType) -> T.Tuple[T.Any, ...]:
         if type_ is DependencyCacheType.PKG_CONFIG:
             return tuple(self.__builtins_per_machine[self.__for_machine]['pkg_config_path'].value)
         elif type_ is DependencyCacheType.CMAKE:
@@ -298,7 +297,7 @@ class DependencyCache:
         assert type_ is DependencyCacheType.OTHER, 'Someone forgot to update subkey calculations for a new type'
         return tuple()
 
-    def __iter__(self) -> Iterator['CacheKeyType']:
+    def __iter__(self) -> T.Iterator['CacheKeyType']:
         return self.keys()
 
     def put(self, key: 'CacheKeyType', dep: 'dependencies.Dependency') -> None:
@@ -308,7 +307,7 @@ class DependencyCache:
         subkey = self.__calculate_subkey(t)
         self.__cache[key][subkey] = dep
 
-    def get(self, key: 'CacheKeyType') -> Optional['dependencies.Dependency']:
+    def get(self, key: 'CacheKeyType') -> T.Optional['dependencies.Dependency']:
         """Get a value from the cache.
 
         If there is no cache entry then None will be returned.
@@ -326,14 +325,14 @@ class DependencyCache:
                 pass
         return None
 
-    def values(self) -> Iterator['dependencies.Dependency']:
+    def values(self) -> T.Iterator['dependencies.Dependency']:
         for c in self.__cache.values():
             yield from c.values()
 
-    def keys(self) -> Iterator['CacheKeyType']:
+    def keys(self) -> T.Iterator['CacheKeyType']:
         return iter(self.__cache.keys())
 
-    def items(self) -> Iterator[Tuple['CacheKeyType', List['dependencies.Dependency']]]:
+    def items(self) -> T.Iterator[T.Tuple['CacheKeyType', T.List['dependencies.Dependency']]]:
         for k, v in self.__cache.items():
             vs = []
             for t in v.types:
@@ -346,7 +345,7 @@ class DependencyCache:
         self.__cache.clear()
 
 # Can't bind this near the class method it seems, sadly.
-_V = TypeVar('_V')
+_V = T.TypeVar('_V')
 
 # This class contains all data that must persist over multiple
 # invocations of Meson. It is roughly the same thing as
@@ -367,11 +366,12 @@ class CoreData:
         self.install_guid = str(uuid.uuid4()).upper()
         self.target_guids = {}
         self.version = version
-        self.init_builtins()
-        self.backend_options = {} # : Dict[str, UserOption]
-        self.user_options = {} # : Dict[str, UserOption]
+        self.builtins = {} # : OptionDictType
+        self.builtins_per_machine = PerMachine({}, {})
+        self.backend_options = {} # : OptionDictType
+        self.user_options = {} # : OptionDictType
         self.compiler_options = PerMachine({}, {})
-        self.base_options = {} # : Dict[str, UserOption]
+        self.base_options = {} # : OptionDictType
         self.cross_files = self.__load_config_files(options, scratch_dir, 'cross')
         self.compilers = PerMachine(OrderedDict(), OrderedDict())
 
@@ -381,10 +381,10 @@ class CoreData:
         self.compiler_check_cache = OrderedDict()
         # Only to print a warning if it changes between Meson invocations.
         self.config_files = self.__load_config_files(options, scratch_dir, 'native')
-        self.libdir_cross_fixup()
+        self.init_builtins('')
 
     @staticmethod
-    def __load_config_files(options: argparse.Namespace, scratch_dir: str, ftype: str) -> List[str]:
+    def __load_config_files(options: argparse.Namespace, scratch_dir: str, ftype: str) -> T.List[str]:
         # Need to try and make the passed filenames absolute because when the
         # files are parsed later we'll have chdir()d.
         if ftype == 'cross':
@@ -395,9 +395,9 @@ class CoreData:
         if not filenames:
             return []
 
-        found_invalid = []  # type: List[str]
-        missing = []        # type: List[str]
-        real = []           # type: List[str]
+        found_invalid = []  # type: T.List[str]
+        missing = []        # type: T.List[str]
+        real = []           # type: T.List[str]
         for i, f in enumerate(filenames):
             f = os.path.expanduser(os.path.expandvars(f))
             if os.path.exists(f):
@@ -466,7 +466,7 @@ class CoreData:
                 prefix = prefix[:-1]
         return prefix
 
-    def sanitize_dir_option_value(self, prefix, option, value):
+    def sanitize_dir_option_value(self, prefix: str, option: str, value: T.Any) -> T.Any:
         '''
         If the option is an installation directory option and the value is an
         absolute path, check that it resides within prefix and return the value
@@ -474,32 +474,52 @@ class CoreData:
 
         This way everyone can do f.ex, get_option('libdir') and be sure to get
         the library directory relative to prefix.
+
+        .as_posix() keeps the posix-like file seperators Meson uses.
         '''
-        if option.endswith('dir') and os.path.isabs(value) and \
+        try:
+            value = PurePath(value)
+        except TypeError:
+            return value
+        if option.endswith('dir') and value.is_absolute() and \
            option not in builtin_dir_noprefix_options:
             # Value must be a subdir of the prefix
             # commonpath will always return a path in the native format, so we
             # must use pathlib.PurePath to do the same conversion before
             # comparing.
-            if os.path.commonpath([value, prefix]) != str(PurePath(prefix)):
-                m = 'The value of the {!r} option is {!r} which must be a ' \
-                    'subdir of the prefix {!r}.\nNote that if you pass a ' \
-                    'relative path, it is assumed to be a subdir of prefix.'
-                raise MesonException(m.format(option, value, prefix))
-            # Convert path to be relative to prefix
-            skip = len(prefix) + 1
-            value = value[skip:]
-        return value
+            msg = ('The value of the {!r} option is {!r} which must be a '
+                   'subdir of the prefix {!r}.\nNote that if you pass a '
+                   'relative path, it is assumed to be a subdir of prefix.')
+            # os.path.commonpath doesn't understand case-insensitive filesystems,
+            # but PurePath().relative_to() does.
+            try:
+                value = value.relative_to(prefix)
+            except ValueError:
+                raise MesonException(msg.format(option, value, prefix))
+            if '..' in str(value):
+                raise MesonException(msg.format(option, value, prefix))
+        return value.as_posix()
 
-    def init_builtins(self):
+    def init_builtins(self, subproject: str):
         # Create builtin options with default values
-        self.builtins = {}
         for key, opt in builtin_options.items():
-            self.builtins[key] = opt.init_option()
-        self.builtins_per_machine = PerMachine({}, {})
+            self.add_builtin_option(self.builtins, key, opt, subproject)
         for for_machine in iter(MachineChoice):
             for key, opt in builtin_options_per_machine.items():
-                self.builtins_per_machine[for_machine][key] = opt.init_option()
+                self.add_builtin_option(self.builtins_per_machine[for_machine], key, opt, subproject)
+        self.libdir_cross_fixup()
+
+    def add_builtin_option(self, opts_map, key, opt, subproject):
+        if subproject:
+            if opt.yielding:
+                # This option is global and not per-subproject
+                return
+            optname = subproject + ':' + key
+            value = opts_map[key].value
+        else:
+            optname = key
+            value = None
+        opts_map[optname] = opt.init_option(key, value, default_prefix())
 
     def init_backend_options(self, backend_name):
         if backend_name == 'ninja':
@@ -507,22 +527,27 @@ class CoreData:
                 UserIntegerOption(
                     'Maximum number of linker processes to run or 0 for no '
                     'limit',
-                    0, None, 0)
+                    (0, None, 0))
         elif backend_name.startswith('vs'):
             self.backend_options['backend_startup_project'] = \
                 UserStringOption(
                     'Default project to execute in Visual Studio',
                     '')
 
-    def get_builtin_option(self, optname):
+    def get_builtin_option(self, optname, subproject=''):
+        raw_optname = optname
+        if subproject:
+            optname = subproject + ':' + optname
         for opts in self._get_all_builtin_options():
             v = opts.get(optname)
+            if v is None or v.yielding:
+                v = opts.get(raw_optname)
             if v is None:
                 continue
-            if optname == 'wrap_mode':
+            if raw_optname == 'wrap_mode':
                 return WrapMode.from_string(v.value)
             return v.value
-        raise RuntimeError('Tried to get unknown builtin option %s.' % optname)
+        raise RuntimeError('Tried to get unknown builtin option %s.' % raw_optname)
 
     def _try_set_builtin_option(self, optname, value):
         for opts in self._get_all_builtin_options():
@@ -591,8 +616,8 @@ class CoreData:
 
     @staticmethod
     def get_prefixed_options_per_machine(
-        options_per_machine # : PerMachine[Dict[str, _V]]]
-    ) -> Iterable[Dict[str, _V]]:
+        options_per_machine # : PerMachine[T.Dict[str, _V]]]
+    ) -> T.Iterable[T.Dict[str, _V]]:
         for for_machine in iter(MachineChoice):
             prefix = for_machine.get_prefix()
             yield {
@@ -600,17 +625,17 @@ class CoreData:
                 for k, v in options_per_machine[for_machine].items()
             }
 
-    def _get_all_nonbuiltin_options(self) -> Iterable[Dict[str, UserOption]]:
+    def _get_all_nonbuiltin_options(self) -> T.Iterable[T.Dict[str, UserOption]]:
         yield self.backend_options
         yield self.user_options
         yield from self.get_prefixed_options_per_machine(self.compiler_options)
         yield self.base_options
 
-    def _get_all_builtin_options(self) -> Dict[str, UserOption]:
+    def _get_all_builtin_options(self) -> T.Dict[str, UserOption]:
         yield from self.get_prefixed_options_per_machine(self.builtins_per_machine)
         yield self.builtins
 
-    def get_all_options(self) -> Dict[str, UserOption]:
+    def get_all_options(self) -> T.Dict[str, UserOption]:
         yield from self._get_all_nonbuiltin_options()
         yield from self._get_all_builtin_options()
 
@@ -644,7 +669,7 @@ class CoreData:
         return len(self.cross_files) > 0
 
     def strip_build_option_names(self, options):
-        res = {}
+        res = OrderedDict()
         for k, v in options.items():
             if k.startswith('build.'):
                 k = k.split('.', 1)[1]
@@ -688,46 +713,75 @@ class CoreData:
             unknown_options = ', '.join(sorted(unknown_options))
             sub = 'In subproject {}: '.format(subproject) if subproject else ''
             mlog.warning('{}Unknown options: "{}"'.format(sub, unknown_options))
+            mlog.log('The value of new options can be set with:')
+            mlog.log(mlog.bold('meson setup <builddir> --reconfigure -Dnew_option=new_value ...'))
         if not self.is_cross_build():
             self.copy_build_options_from_regular_ones()
 
     def set_default_options(self, default_options, subproject, env):
-        # Set defaults first from conf files (cross or native), then
-        # override them as nec as necessary.
-        for k, v in env.paths.host:
-            if v is not None:
-                env.cmd_line_options.setdefault(k, v)
-
-        # Set default options as if they were passed to the command line.
-        # Subprojects can only define default for user options.
+        # Warn if the user is using two different ways of setting build-type
+        # options that override each other
+        if 'buildtype' in env.cmd_line_options and \
+           ('optimization' in env.cmd_line_options or 'debug' in env.cmd_line_options):
+            mlog.warning('Recommend using either -Dbuildtype or -Doptimization + -Ddebug. '
+                         'Using both is redundant since they override each other. '
+                         'See: https://mesonbuild.com/Builtin-options.html#build-type-options')
+        cmd_line_options = OrderedDict()
+        # Set project default_options as if they were passed to the cmdline.
+        # Subprojects can only define default for user options and not yielding
+        # builtin option.
         from . import optinterpreter
         for k, v in default_options.items():
             if subproject:
-                if optinterpreter.is_invalid_name(k, log=False):
+                if (k not in builtin_options or builtin_options[k].yielding) \
+                        and optinterpreter.is_invalid_name(k, log=False):
                     continue
                 k = subproject + ':' + k
-            env.cmd_line_options.setdefault(k, v)
+            cmd_line_options[k] = v
+
+        # Override project default_options using conf files (cross or native)
+        for k, v in env.paths.host:
+            if v is not None:
+                cmd_line_options[k] = v
+
+        # Override all the above defaults using the command-line arguments
+        # actually passed to us
+        cmd_line_options.update(env.cmd_line_options)
+        env.cmd_line_options = cmd_line_options
 
         # Create a subset of cmd_line_options, keeping only options for this
         # subproject. Also take builtin options if it's the main project.
         # Language and backend specific options will be set later when adding
         # languages and setting the backend (builtin options must be set first
         # to know which backend we'll use).
-        options = {}
+        options = OrderedDict()
 
         # Some options default to environment variables if they are
         # unset, set those now. These will either be overwritten
         # below, or they won't. These should only be set on the first run.
-        if env.first_invocation:
-            p_env = os.environ.get('PKG_CONFIG_PATH')
-            if p_env:
+        for for_machine in MachineChoice:
+            p_env_pair = get_env_var_pair(for_machine, self.is_cross_build(), 'PKG_CONFIG_PATH')
+            if p_env_pair is not None:
+                p_env_var, p_env = p_env_pair
+
                 # PKG_CONFIG_PATH may contain duplicates, which must be
                 # removed, else a duplicates-in-array-option warning arises.
-                pkg_config_paths = []
-                for k in p_env.split(':'):
-                    if k not in pkg_config_paths:
-                        pkg_config_paths.append(k)
-                options['pkg_config_path'] = pkg_config_paths
+                p_list = list(OrderedSet(p_env.split(':')))
+
+                key = 'pkg_config_path'
+                if for_machine == MachineChoice.BUILD:
+                    key = 'build.' + key
+
+                if env.first_invocation:
+                    options[key] = p_list
+                elif options.get(key, []) != p_list:
+                    mlog.warning(
+                        p_env_var +
+                        ' environment variable has changed '
+                        'between configurations, meson ignores this. '
+                        'Use -Dpkg_config_path to change pkg-config search '
+                        'path instead.'
+                    )
 
         for k, v in env.cmd_line_options.items():
             if subproject:
@@ -744,13 +798,18 @@ class CoreData:
 
         self.set_options(options, subproject=subproject)
 
-    def add_lang_args(self, lang: str, comp: Type['Compiler'],
+    def add_lang_args(self, lang: str, comp: T.Type['Compiler'],
                       for_machine: MachineChoice, env: 'Environment') -> None:
         """Add global language arguments that are needed before compiler/linker detection."""
         from .compilers import compilers
 
         optprefix = lang + '_'
-        for k, o in compilers.get_global_options(lang, comp, env.properties[for_machine]).items():
+        for k, o in compilers.get_global_options(
+                lang,
+                comp,
+                for_machine,
+                env.is_cross_build(),
+                env.properties[for_machine]).items():
             if not k.startswith(optprefix):
                 raise MesonException('Internal error, %s has incorrect prefix.' % k)
             # prefixed compiler options affect just this machine
@@ -759,7 +818,7 @@ class CoreData:
                 o.set_value(env.cmd_line_options[opt_prefix + k])
             self.compiler_options[for_machine].setdefault(k, o)
 
-    def process_new_compiler(self, lang: str, comp: Type['Compiler'], env: 'Environment') -> None:
+    def process_new_compiler(self, lang: str, comp: T.Type['Compiler'], env: 'Environment') -> None:
         from . import compilers
 
         self.compilers[comp.for_machine][lang] = comp
@@ -822,17 +881,20 @@ def read_cmd_line_file(build_dir, options):
         # literal_eval to get it into the list of strings.
         options.native_file = ast.literal_eval(properties.get('native_file', '[]'))
 
+def cmd_line_options_to_string(options):
+    return {k: str(v) for k, v in options.cmd_line_options.items()}
+
 def write_cmd_line_file(build_dir, options):
     filename = get_cmd_line_file(build_dir)
     config = CmdLineFileParser()
 
-    properties = {}
+    properties = OrderedDict()
     if options.cross_file:
         properties['cross_file'] = options.cross_file
     if options.native_file:
         properties['native_file'] = options.native_file
 
-    config['options'] = options.cmd_line_options
+    config['options'] = cmd_line_options_to_string(options)
     config['properties'] = properties
     with open(filename, 'w') as f:
         config.write(f)
@@ -841,7 +903,7 @@ def update_cmd_line_file(build_dir, options):
     filename = get_cmd_line_file(build_dir)
     config = CmdLineFileParser()
     config.read(filename)
-    config['options'].update(options.cmd_line_options)
+    config['options'].update(cmd_line_options_to_string(options))
     with open(filename, 'w') as f:
         config.write(f)
 
@@ -906,7 +968,7 @@ def register_builtin_arguments(parser):
                         help='Set the value of an option, can be used several times to set multiple options.')
 
 def create_options_dict(options):
-    result = {}
+    result = OrderedDict()
     for o in options:
         try:
             (key, value) = o.split('=', 1)
@@ -934,38 +996,41 @@ def parse_cmd_line_options(args):
             delattr(args, name)
 
 
-_U = TypeVar('_U', bound=UserOption[_T])
+_U = T.TypeVar('_U', bound=UserOption[_T])
 
-class BuiltinOption(Generic[_T, _U]):
+class BuiltinOption(T.Generic[_T, _U]):
 
     """Class for a builtin option type.
 
-    Currently doesn't support UserIntegerOption, or a few other cases.
+    There are some cases that are not fully supported yet.
     """
 
-    def __init__(self, opt_type: Type[_U], description: str, default: Any, yielding: Optional[bool] = None, *,
-                 choices: Any = None):
+    def __init__(self, opt_type: T.Type[_U], description: str, default: T.Any, yielding: bool = True, *,
+                 choices: T.Any = None):
         self.opt_type = opt_type
         self.description = description
         self.default = default
         self.choices = choices
         self.yielding = yielding
 
-    def init_option(self) -> _U:
+    def init_option(self, name: str, value: T.Optional[T.Any], prefix: str) -> _U:
         """Create an instance of opt_type and return it."""
-        keywords = {'yielding': self.yielding, 'value': self.default}
+        if value is None:
+            value = self.prefixed_default(name, prefix)
+        keywords = {'yielding': self.yielding, 'value': value}
         if self.choices:
             keywords['choices'] = self.choices
         return self.opt_type(self.description, **keywords)
 
-    def _argparse_action(self) -> Optional[str]:
-        if self.default is True:
-            return 'store_false'
-        elif self.default is False:
+    def _argparse_action(self) -> T.Optional[str]:
+        # If the type is a boolean, the presence of the argument in --foo form
+        # is to enable it. Disabling happens by using -Dfoo=false, which is
+        # parsed under `args.projectoptions` and does not hit this codepath.
+        if isinstance(self.default, bool):
             return 'store_true'
         return None
 
-    def _argparse_choices(self) -> Any:
+    def _argparse_choices(self) -> T.Any:
         if self.opt_type is UserBooleanOption:
             return [True, False]
         elif self.opt_type is UserFeatureOption:
@@ -979,7 +1044,7 @@ class BuiltinOption(Generic[_T, _U]):
         else:
             return '--' + name.replace('_', '-')
 
-    def prefixed_default(self, name: str, prefix: str = '') -> Any:
+    def prefixed_default(self, name: str, prefix: str = '') -> T.Any:
         if self.opt_type in [UserComboOption, UserIntegerOption]:
             return self.default
         try:
@@ -989,7 +1054,7 @@ class BuiltinOption(Generic[_T, _U]):
         return self.default
 
     def add_to_argparse(self, name: str, parser: argparse.ArgumentParser, prefix: str, help_suffix: str) -> None:
-        kwargs = {}
+        kwargs = OrderedDict()
 
         c = self._argparse_choices()
         b = self._argparse_action()
@@ -1028,7 +1093,8 @@ builtin_options = OrderedDict([
     ('buildtype',       BuiltinOption(UserComboOption, 'Build type to use', 'debug',
                                       choices=['plain', 'debug', 'debugoptimized', 'release', 'minsize', 'custom'])),
     ('debug',           BuiltinOption(UserBooleanOption, 'Debug', True)),
-    ('default_library', BuiltinOption(UserComboOption, 'Default library type', 'shared', choices=['shared', 'static', 'both'])),
+    ('default_library', BuiltinOption(UserComboOption, 'Default library type', 'shared', choices=['shared', 'static', 'both'],
+                                      yielding=False)),
     ('errorlogs',       BuiltinOption(UserBooleanOption, "Whether to print the logs from failing tests", True)),
     ('install_umask',   BuiltinOption(UserUmaskOption, 'Default umask to apply on permissions of installed files', '022')),
     ('layout',          BuiltinOption(UserComboOption, 'Build directory layout', 'mirror', choices=['mirror', 'flat'])),
@@ -1036,8 +1102,9 @@ builtin_options = OrderedDict([
     ('stdsplit',        BuiltinOption(UserBooleanOption, 'Split stdout and stderr in test logs', True)),
     ('strip',           BuiltinOption(UserBooleanOption, 'Strip targets on install', False)),
     ('unity',           BuiltinOption(UserComboOption, 'Unity build', 'off', choices=['on', 'off', 'subprojects'])),
+    ('unity_size',      BuiltinOption(UserIntegerOption, 'Unity block size', (2, None, 4))),
     ('warning_level',   BuiltinOption(UserComboOption, 'Compiler warning level to use', '1', choices=['0', '1', '2', '3'])),
-    ('werror',          BuiltinOption(UserBooleanOption, 'Treat warnings as errors', False)),
+    ('werror',          BuiltinOption(UserBooleanOption, 'Treat warnings as errors', False, yielding=False)),
     ('wrap_mode',       BuiltinOption(UserComboOption, 'Wrap mode', 'default', choices=['default', 'nofallback', 'nodownload', 'forcefallback'])),
 ])
 
